open util/ordering[House]

sig House {
    color: Color,
    nationality: Nationality,
    drink: Drink,
    cigarette: Cigarette,
    pet: Pet
}

enum Color { red, green, yellow, blue, white }
enum Nationality { Englishman, Swede, Dane, German, Norwegian }
enum Drink { tea, coffee, milk, beer, water }
enum Cigarette { Pall_Mall, Dunhills, Blend, Blue_Masters, Prince }
enum Pet { dog, bird, horse, cat, fish }

fact constraints {
    // There are five houses, each of a different color.
    some disj h1, h2, h3, h4, h5: House | h1.color = red and h2.color = green and h3.color = yellow and h4.color = blue and h5.color = white
    // In each house lives a man of a different nationality.
    no disj h,h': House | h.nationality = h'.nationality
    // 1. The Englishman lives in the red house.
    some h: House | (h.nationality = Englishman) and (h.color = red)
    // 2. The Swede keeps dogs.
    some h: House | (h.nationality = Swede) and (h.pet = dog)
    // 3. The Dane drinks tea.
    some h: House | (h.nationality = Dane) and (h.drink = tea)
    // 4. The green house is just to the left of the white one.
    some disj h, h': House | (h.color = green) and (h'.color = white) and (h'.prev = h)
    // 5. The owner of the green house drinks coffee.
    some h: House | (h.color = green) and (h.drink = coffee)
    // 6. The Pall Mall smoker keeps birds.
    some h: House | (h.cigarette = Pall_Mall) and (h.pet = bird)
    // 7. The owner of the yellow house smokes Dunhills.
    some h: House | (h.color = yellow) and (h.cigarette = Dunhills)
    // 8.  The man in the center house drinks milk.
    some h: House | (some h.prev.prev) and (some h.next.next) and (h.drink = milk)
    // 9. The Norwegian lives in the first house.
    some h: House | (h = first) and (h.nationality = Norwegian)
    // 10. The Blend smoker has a neighbor who keeps cats.
    some disj h,h': House | (h.cigarette = Blend) and (h'.pet = cat) and  ((h.prev = h') or (h.next = h')) 
    // 11. The man who smokes Blue Masters drinks beer.
    some h: House | (h.cigarette = Blue_Masters) and (h.drink = beer)
    // 12. The man who keeps horses lives next to the Dunhill smoker.
    some disj h,h': House | (h.pet = horse) and (h'.cigarette = Dunhills) and ((h.next = h') or (h.prev = h'))
    // 13. The German smokes Prince.
    some h: House | (h.nationality = German) and (h.cigarette = Prince)
    // 14. The Norwegian lives next to the blue house.
    some disj h,h': House | (h.nationality = Norwegian) and (h'.color= blue) and (h.next = h')
    // 15. The Blend smoker has a neighbor who drinks water.
    some disj h,h': House | (h.cigarette = Blend) and (h'.drink = water) and ((h.next = h') or (h.prev = h'))
}

run {} for 5
