open Traffic_Lights
open Intersections

// For each intersection i, if the intersection to the west 
// has its east-facing traffic light green at time t-1, then i's
// east-facing traffic light (if there is one) must be green
// at time t. Ditto for the other directions. And, of course, 
// for every time step.
fact Lights_are_synchronized {
    all t: Time - first |
        all i: Intersection |
            all d: i.connections.Intersection | 
                i.connections[d].lights[OppositeDirection.reverse[d]].color.(t.prev) = green =>
                    some i.connections[OppositeDirection.reverse[d]] =>
                        i.lights[OppositeDirection.reverse[d]].color.t = green
}

// At each intersection, at least one traffic light
// changes color at each time step.
fact Constantly_changing_lights {
    all t: Time - first | 
        all i: Intersection |
            some d: Direction |
                i.lights[d].color.(t.prev) != i.lights[d].color.t
}

// Traffic lights change color in the proper 
// sequence: red -> yellow -> green -> red
fact Lights_change_color_in_the_proper_sequence {
    all t: Time - first | 
        all light: Traffic_Light |  
            light.color.(t.prev) != light.color.t => 
               light.color.t = Color_Sequence.sequence[light.color.(t.prev)]
}

// At each intersection there is at most one 
// traffic light that is not red.
fact At_most_one_non_red_light {
    all t: Time |
        all i: Intersection |
            no disj d, d': Direction | 
               ((i.lights[d].color.t = green) or (i.lights[d].color.t = yellow)) and
               ((i.lights[d'].color.t = green) or (i.lights[d'].color.t = yellow))
}

run {} for 10


