// The store maintains a record
// for each customer.
sig StoreRecord {
    customer: disj Customer,
    userId:  disj UserId,
    password:  disj Password,
    token: disj Token
} {
    (userId = customer.userId) and
    (password = customer.password)
}

fact A_StoreRecord_for_each_customer {
    all c: Customer | one s: StoreRecord | s.customer = c
}

sig UserId, Password, Token {}

abstract sig Customer {
    userId:  UserId,
    password:  Password,
    token: lone Token,
    knows: set Password
}
one sig Alice extends Customer {}
one sig Eve extends Customer {}

fun Login [customerUserId: UserId, customerPassword: Password]: Token {
    {storeRecord: StoreRecord | (storeRecord.userId = customerUserId) and (storeRecord.password = customerPassword)}.token
}

pred logsOnAs[c1, c2 : Customer] {
	some p : Password | 
		some Login[c2.userId, p] and
		p in c1.knows
}

// Should generate an instance
run EveLogsOn1 {
	// Eve knows Alice's password
	Eve.knows = Eve.password + Alice.password
	Eve.logsOnAs[Alice]
}

// Should not generate an instance
run EveLogsOn2 {
	// Eve only knows its own password
	Eve.knows = Eve.password
	Eve.logsOnAs[Alice]
}

// Should not generate an instance
run EveCannotLogOn {
	Alice.password != Eve.password
	Alice.userId != Eve.userId
	// Eve only knows its own password
	Eve.knows = Eve.password
	Eve.logsOnAs[Alice]	
}
