module Asymmetric

open Inverse
open Complement
open Common

/***************************
        ASYMMETRIC PROPERTY
****************************/

// Given a set A and a relation R in A, R is asymmetric
// if it is never the case that for any (x, y) in R, the
// pair (y, x) is in R.

sig A {}

one sig Test {
    R: A -> A
} {
    (R & ~R) = none -> none  // Constrain R to be asymmetric
}

pred Show {
    #A > 2
}
run Show

assert isAsymmetric {
    asymmetric  [A, Test.R]
    // all x, y: A | ((x -> y) in Test.R) => ((y -> x) not in Test.R)
}
check isAsymmetric

// The inverse of an asymmetric relation is asymmetric.
assert inverseIsAsymmetric {
   asymmetric [A, inverse[Test.R]]
}
check inverseIsAsymmetric

// The complement of an asymmetric relation is non-symmetric.
// Counterexample! Let A = {} and R = {}
assert complementIsNonsymmetric {
   nonsymmetric [A, complement[A, Test.R]]
}
check complementIsNonsymmetric
