module Nontransitive

open Inverse
open Complement
open Common

/***************************
      NONTRANSITIVE PROPERTY
****************************/

// If a relation fails to meet the definition of transitivity, 
// it is nontransitive. So, there may be transitivity in the 
// relation, but there is at least one (x,y) and (y,z) that 
// does not have (x,z).

sig A {}

one sig Test {
    R: A -> A
} {
   R.R & R != R.R   // Constrain R to be non-transitive 
}

pred Show {
    #A > 2
}
run Show

assert isNontransitive {
    nontransitive [A, inverse[Test.R]]
    // some  x, y, z: A | (((x -> y) in Test.R) and ((y -> z) in Test.R)) => ((x -> z) not in Test.R)
}
check isNontransitive
