module Ordered

open Common

sig A {}

one sig Ord {
    First: A,
    Next: A -> A
} {
    no x: A | (x -> First) in Next   // First is the first atom in the list
    irreflexive [A, Next]    // Constrain Next to be irreflexive
    intransitive [A, Next]   // Constrain Next to be intransitive 
    asymmetric [A, Next]    // Constrain Next to be asymmetric
    nonconnected [A, Next]    // Constrain Next to be non-connected
    injective [A, Next]   // Constrain Next to be injective
    functional [A, Next]   // Constrain Next to be functional
    #Next = minus[#A, 1]  // Constraint Next to contain all atoms in A
}

fun first: one A { Ord.First }
fun next : A->A { Ord.Next }
fun last: one A { A - (next.A) }
fun prev : A->A { ~(Ord.Next) }

pred Show {
    #A > 2
}
run Show

assert correct {
    let mynext = Ord.Next |
    let myprev = ~mynext | {
        // Each list item has at most on next item, at most one previous item, and
        // the item is not in its list of successors.
        ( all b: A | (lone b.next) && (lone b.prev) && (b !in b.^mynext) )
        // There is no item before first and no item after last.
        ( (no first.prev) && (no last.next) )
        // For all items between first and last, there is exactly one previous item and
        // one next item.
        ( all b: A | (b!=first && b!=last) => (one b.prev && one b.next) )
        // If there is *not* exactly one member in set A, then there must
        // be exactly one value returned by the "first" function and exactly
        // one value returned by the "last" function and the value returned
        // by "first" must not equal the value returned by "last" and there is
        // exactly one value returned by "first.next" and exactly one value
        // value returned by "last.prev"
        ( !one A => (one first && one last && first!=last && one first.next && one last.prev) )
        // If there *is* exactly one member in set A, then the value returned
        // by the "first" function must equal A and the value returned by the
        // "last" function must equal A and there is no prev and no next.
        ( one A => (first=A && last=A && no myprev && no mynext) )
        // prev equals next, reversed 
        ( myprev=~mynext )
        // The set A equals all the values reachable from the value returned by
        // the "first" function, including the value
        ( A = first.*mynext )
        // For every value a and b in set A, "a" is either in the set reachable by b
        // or "a" is in the set reachable by b when looking in the opposite direction
        // (the prev direction)
        (all disj a,b: A | a in b.^mynext or a in b.^myprev)
        // For every value a and b in set A, a cannot be in both the set reachable
        // by b and the set reachable by b when looking in the opposite direction
        // (the prev direction)
        (no disj a,b: A | a in b.^mynext and a in b.^myprev)
        // For every value a, b, c in set A, if b is in the set reachable by "a" and
        // c is in the set reachable by b, then c is in the set reachable by "a"
        (all disj a,b,c: A | (b in a.^mynext and c in b.^mynext) => (c in a.^mynext))
    }
}
check correct
