module Ordering

open Inverse
open Complement
open Common

/***************************
                    ORDER
****************************/

// An order is a binary relation which is transitive and 
// in addition either (i) reflexive and antisymmetric or 
// else (ii) irreflexive and asymmetric. The former are 
// weak orders; the latter are strict (or strong). 

abstract sig Letter {}
one sig a extends Letter {}
one sig b extends Letter {}
one sig c extends Letter {}

one sig Test {
    R: Letter -> Letter
} 

pred transitive_reflexive (A: univ, R: univ->univ) {
    transitive [A, R]
    reflexive [A, R]
}

pred transitive_reflexive_antisymmetric (A: univ, R: univ->univ) {
    transitive [A, R]
    reflexive [A, R] 
    antisymmetric [A, R]
}

pred IncludeEveryLetter {
    Letter in (Test.R[Letter] + Test.R.Letter)
}

pred ConnectEveryLetter {
    connected [Letter, Test.R]
}

fact {
    IncludeEveryLetter
    ConnectEveryLetter
}

pred Show { 
    transitive [Letter, Test.R]
    //transitive_reflexive[Letter, Test.R]
    //transitive_reflexive_antisymmetric[Letter, Test.R]
}
run Show

// transitive-reflexive allows back-arrows whereas
// transitive-reflexive-antisymmetric does not.
assert Equal {
    transitive_reflexive[Letter, Test.R] iff transitive_reflexive_antisymmetric[Letter, Test.R]
}
check Equal

