module Reflexive

open Inverse
open Complement
open Common

/***************************
            REFLEXIVE PROPERTY
****************************/

// Given a set A and a relation R in A, R is reflexive 
// if and only if all the ordered pairs of the form (x, x) 
// are in R for every x in A.

sig A {}

one sig Test {
    R: A -> A
} {
    (A <: iden) in R  // Constrain R to be reflexive
}

pred Show {
    #A > 2
}
run Show

assert isReflexive {
    reflexive [A, Test.R]
    //all x: A | (x -> x) in Test.R
}
check isReflexive

// The inverse of a reflexive relation is reflexive.
assert inverseIsReflexive {
    reflexive [A, inverse[Test.R]]
}
check inverseIsReflexive

// The complement of a reflexive relation is irreflexive.
assert complementIsIrreflexive {
    irreflexive [A, complement[A, Test.R]]
}
check complementIsIrreflexive

