module Nonsymmetric

open Inverse
open Complement
open Common

/***************************
          SYMMETRIC PROPERTY
****************************/

// Given a set A and a relation R in A, R is symmetric
// if and only if for every ordered pair (x, y) in R, the
// pair (y, x) is also in R.

sig A {}

one sig Test {
    R: A -> A
} {
    ~R in R  // Constrain R to be symmetric
}

pred Show {
    #A > 2
}
run Show

assert isSymmetric {
    symmetric  [A, Test.R]
    // all disj x, y: A | ((x -> y) in Test.R) => ((y -> x) in Test.R)
}
check isSymmetric

// The inverse of a symmetric relation is symmetric.
assert inverseIsSymmetric {
   symmetric [A, inverse[Test.R]]
}
check inverseIsSymmetric

// The complement of a symmetric relation is symmetric.
assert complementIsSymmetric {
   symmetric [A, complement[A, Test.R]]
}
check complementIsSymmetric
