open util/ordering[House]

sig House {
    color: Color,
    nationality: Nationality,
    sport: Sport,
    animal: Animal
}

enum Color { black, red, white, blue }
enum Nationality { Irish, American, British, Canadian }
enum Sport { bowling, swimming, handball, tennis }
enum Animal { butterfly, dolphin, horse, turtle }

fact constraints {
    // There are four houses, each of a different color.
    some disj h1, h2, h3, h4: House | h1.color = black and h2.color = red and h3.color = white and h4.color = blue 
    // In each house lives a man of a different nationality.
    no disj h,h': House | h.nationality = h'.nationality
    // Each person likes a different sport.
    no disj h,h': House | h.sport = h'.sport
    // Each person likes a different animal.
    no disj h,h': House | h.animal = h'.animal
    // 1. There are two houses between the person who likes 
    //      bowling and the person who likes swimming.
    some disj h,h': House  | (h.sport = bowling) and (h'.sport = swimming) and
                                                   ((h.next.next.next = h') or (h.prev.prev.prev = h'))
    // 2. There is one house between the Irish and the person 
    //      who likes handball on the left.
    some disj h,h': House | (h.nationality = Irish) and (h'.sport = handball) and
                                                  (h.prev.prev = h')
    // 3. The second house is black.
    first.next.color = black
    // 4.	There is one house between the person who likes 
    //       horses and the red house on the right.
    some disj h, h': House | (h.animal= horse) and (h'.color = red) and 
                                                   (h.next.next = h')
    // 5.	The American lives directly to the left of the person who likes turtles.
    some disj h, h': House | (h.nationality = American) and (h'.animal = turtle) and 
                                                   (h.next = h')
    // 6.	There are two houses between the person who likes 
    //       horses and the person who likes butterflies on the right.
    some disj h, h': House | (h.animal= horse) and (h'.animal = butterfly) and 
                                                   (h.next.next.next = h')
    // 7.	The person who likes bowling lives somewhere to the 
   //        right of the person who likes tennis.
    some disj h, h': House | (h.sport = bowling) and (h'.sport = tennis) and 
                                                   (h in h'.nexts)
    // 8.	There is one house between the person who likes handball 
    //       and the white house on the right.
    some disj h, h': House | (h.sport = handball) and (h'.color = white) and 
                                                   (h.next.next = h')
    // 9.	The British lives in the first house.
    first.nationality = British
}

run {} for 4
